"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloNextSSRVisitor = void 0;
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const auto_bind_1 = __importDefault(require("auto-bind"));
class ApolloNextSSRVisitor extends visitor_plugin_common_1.ClientSideBaseVisitor {
    constructor(schema, fragments, rawConfig, documents) {
        super(schema, fragments, rawConfig, {
            apolloReactCommonImportFrom: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloReactCommonImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "@apollo/react-common"),
            apolloReactHooksImportFrom: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloReactHooksImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "@apollo/react-hooks"),
            apolloImportFrom: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloImportFrom, rawConfig.reactApolloVersion === 3 ? "@apollo/client" : "apollo-client"),
            reactApolloVersion: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.reactApolloVersion, 2),
            excludePatterns: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.excludePatterns, null),
            excludePatternsOptions: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.excludePatternsOptions, ""),
            replacePage: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.replacePage, true),
            replaceQuery: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.replaceQuery, true),
            pre: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.pre, ""),
            post: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.post, ""),
            customImports: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.customImports, null),
            withHOC: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.withHOC, true),
            withHooks: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.withHooks, false),
            apolloClientInstanceImport: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloClientInstanceImport, ""),
            contextType: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.contextType, "any"),
            contextTypeRequired: (0, visitor_plugin_common_1.getConfigValue)(!!rawConfig.contextTypeRequired, false),
            apolloCacheImportFrom: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloCacheImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "apollo-cache-inmemory"),
            apolloStateKey: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.apolloStateKey, 'apolloState'),
            reactImport: (0, visitor_plugin_common_1.getConfigValue)(rawConfig.reactImport, `import type React from 'react';`),
        });
        this.imports = new Set();
        this._externalImportPrefix = this.config.importOperationTypesFrom
            ? `${this.config.importOperationTypesFrom}.`
            : "";
        this._documents = documents;
        (0, auto_bind_1.default)(this);
    }
    getImports() {
        if (this.config.withHOC) {
            this.imports.add(`import { NextPage } from 'next';`);
        }
        this.imports.add(`import { NextRouter, useRouter } from 'next/router'`);
        this.imports.add(`import { QueryHookOptions, useQuery } from '${this.config.apolloReactHooksImportFrom}';`);
        this.imports.add(`import * as Apollo from '${this.config.apolloImportFrom}';`);
        this.imports.add(this.config.reactImport);
        if (this.config.apolloClientInstanceImport) {
            this.imports.add(`import { getApolloClient ${this.config.contextType !== 'any' ? ', ' + this.config.contextType : ''}} from '${this.config.apolloClientInstanceImport}';`);
        }
        if (!this.config.apolloClientInstanceImport) {
            this.imports.add(`import type { NormalizedCacheObject } from '${this.config.apolloCacheImportFrom}';`);
        }
        if (this.config.customImports) {
            this.imports.add(this.config.customImports);
        }
        const baseImports = super.getImports();
        const hasOperations = this._collectedOperations.length > 0;
        if (!hasOperations) {
            return baseImports;
        }
        return [...baseImports, ...Array.from(this.imports)];
    }
    getDocumentNodeVariable(documentVariableName) {
        return this.config.documentMode === visitor_plugin_common_1.DocumentMode.external
            ? `Operations.${documentVariableName}`
            : documentVariableName;
    }
    _buildOperationPageQuery(node, documentVariableName, operationResultType, operationVariablesTypes) {
        const operationName = this.convertName(node, {
            useTypesPrefix: false,
        });
        if (node.operation === "mutation" ||
            (this.config.excludePatterns &&
                new RegExp(this.config.excludePatterns, this.config.excludePatternsOptions).test(operationName))) {
            return "";
        }
        let pageOperation = operationName;
        if (this.config.replacePage) {
            pageOperation = pageOperation.replace(/page/i, "");
        }
        if (this.config.replaceQuery) {
            pageOperation = pageOperation.replace(/query/i, "");
        }
        const WrappedComp = `export type Page${pageOperation}Comp = React.FC<{data?: ${operationResultType}, error?: Apollo.ApolloError}>;`;
        const pageQueryString = this.config.withHOC
            ? `export const withPage${pageOperation} = (optionsFunc?: (router: NextRouter)=> QueryHookOptions<${operationResultType}, ${operationVariablesTypes}>) => (WrappedComponent:Page${pageOperation}Comp) : NextPage  => (props) => {
                const router = useRouter()
                const options = optionsFunc ? optionsFunc(router) : {};
                const {data, error } = useQuery(${this.getDocumentNodeVariable(documentVariableName)}, options)    
                return <WrappedComponent {...props} data={data} error={error} /> ;
                   
            }; `
            : "";
        const pageHook = this.config.withHooks
            ? `export const use${pageOperation} = (
  optionsFunc?: (router: NextRouter)=> QueryHookOptions<${operationResultType}, ${operationVariablesTypes}>) => {
  const router = useRouter();
  const options = optionsFunc ? optionsFunc(router) : {};
  return useQuery(${this.getDocumentNodeVariable(documentVariableName)}, options);
};`
            : "";
        const getSSP = `export async function getServerPage${pageOperation}
    (options: Omit<Apollo.QueryOptions<${operationVariablesTypes}>, 'query'>, ${this.config.apolloClientInstanceImport
            ? `ctx${this.config.contextTypeRequired ? '' : '?'}: ${this.config.contextType}`
            : "apolloClient: Apollo.ApolloClient<NormalizedCacheObject>"} ){
        ${this.config.apolloClientInstanceImport
            ? "const apolloClient = getApolloClient(ctx);"
            : ""}
        
        const data = await apolloClient.query<${operationResultType}>({ ...options, query: ${this.getDocumentNodeVariable(documentVariableName)} });
        
        const apolloState = apolloClient.cache.extract();

        return {
            props: {
                ${this.config.apolloStateKey}: apolloState,
                data: data?.data,
                error: data?.error ?? data?.errors ?? null,
            },
        };
      }`;
        const ssr = `export const ssr${pageOperation} = {
      getServerPage: getServerPage${pageOperation},
      ${this.config.withHOC ? `withPage: withPage${pageOperation},` : ""}
      ${this.config.withHooks ? `usePage: use${pageOperation},` : ""}
    }`;
        return [getSSP, pageHook, WrappedComp, pageQueryString, ssr]
            .filter((a) => a)
            .join("\n");
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        operationResultType = this._externalImportPrefix + operationResultType;
        operationVariablesTypes =
            this._externalImportPrefix + operationVariablesTypes;
        const pageOperation = this._buildOperationPageQuery(node, documentVariableName, operationResultType, operationVariablesTypes);
        return [pageOperation].join("\n");
    }
}
exports.ApolloNextSSRVisitor = ApolloNextSSRVisitor;
